/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.se.xslt.su;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.xml.transform.Source;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.util.ClassLoaderUtil;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.se.xslt.Constants;
import org.ow2.petals.se.xslt.XsltComponent;

/**
 * @author rnaudin
 * 
 */
public class SUManager extends AbstractServiceUnitManager {

    /**
     * Default constructor.
     * 
     * @param component
     */
    public SUManager(AbstractComponent component) {
        super(component);
    }

    @Override
    protected void doDeploy(String serviceUnitName, String suRootPath, Jbi jbiDescriptor)
            throws PEtALSCDKException {
        // control that there is only one Provides section in the SU
        if (jbiDescriptor.getServices().getConsumes().size() != 0) {
            throw new PEtALSCDKException("Consumes section are not supported by this component");
        }

        final List<Provides> providesList = jbiDescriptor.getServices().getProvides();
        if (providesList == null || providesList.size() != 1) {
            throw new PEtALSCDKException("Only one Provides section supported");
        }

        // control that there is the required parameter, as there is for the
        // moment no validation against component level XSD
        final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService(providesList.get(0));
        final ConfigurationExtensions extensions = suDatahandler
                .getConfigurationExtensions(providesList.get(0));
        if (extensions.size() == 0 || extensions.size() > 2) {
            throw new PEtALSCDKException("Provides section must contains only the XSLT parameter '"
                    + Constants.XSL_PATH + "' and can contain the parameter '"
                    + Constants.OUTPUT_ATTACHMENT_NAME + "'");
        }
        final String xslPath = extensions.get(Constants.XSL_PATH);
        if (StringHelper.isNullOrEmpty(xslPath)) {
            throw new PEtALSCDKException("Provides section must contains a XSLT parameter '"
                    + Constants.XSL_PATH + "'");
        }
        if (extensions.size() == 2) {
            final String outputAttachmentName = extensions.get(Constants.OUTPUT_ATTACHMENT_NAME);
            if (StringHelper.isNullOrEmpty(outputAttachmentName)) {
                throw new PEtALSCDKException(
                        "Provides section must contains only the XSLT parameter '"
                                + Constants.XSL_PATH + "' and can contain the parameter '"
                                + Constants.OUTPUT_ATTACHMENT_NAME + "'");
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doInit
     * (java.lang.String, java.lang.String)
     */
    @Override
    protected void doInit(String serviceUnitName, String suRootPath) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final Provides provides = jbiDescriptor.getServices().getProvides().get(0);
        final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService(provides);
        final ConfigurationExtensions extensions = suDatahandler
                .getConfigurationExtensions(provides);
        final String xslPath = extensions.get(Constants.XSL_PATH);
        try {
            this.addTemplate(xslPath, suRootPath, provides.getEndpointName());
        } catch (TransformerConfigurationException e) {
            throw new PEtALSCDKException(e);
        } catch (IOException e) {
            throw new PEtALSCDKException(e);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doShutdown
     * (java.lang.String)
     */
    @Override
    protected void doShutdown(String serviceUnitName) throws PEtALSCDKException {
        final Jbi jbiDescriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
        final String endpointName = jbiDescriptor.getServices().getProvides().get(0)
                .getEndpointName();
        ((XsltComponent) this.component).getTemplates().remove(endpointName);
        ((XsltComponent) this.component).getCustomizedClassLoader().remove(endpointName);
    }

    /**
     * Add the templates to the component map and eventually its customized
     * classloader.
     * 
     * @return
     * @throws IOException
     * @throws TransformerConfigurationException
     * @throws PEtALSCDKException
     */
    private void addTemplate(String xslFileName, String installRoot, String endpointName)
            throws IOException, TransformerConfigurationException, PEtALSCDKException {

        InputStream xslStream;
        // try to get the XSL from the filesystem
        try {
            xslStream = new FileInputStream(new File(installRoot, xslFileName));
        } catch (final FileNotFoundException e) {
            throw new PEtALSCDKException("Failed to retrieve XSL resource", e);
        }

        // check is any Java class containing XSL custom functions or
        // elements are provided
        boolean clazzProvided = false;
        for (final File file : new File(installRoot).listFiles()) {
            if (file.getName().endsWith(".jar")) {
                // check that the jar doesn't contain only a META-INF
                // directory
                final JarFile jar = new JarFile(file);
                final Enumeration<JarEntry> entryEnum = jar.entries();
                while (entryEnum.hasMoreElements()) {
                    final JarEntry jarEntry = entryEnum.nextElement();
                    if (jarEntry.getName().endsWith(".class")) {
                        clazzProvided = true;
                        break;
                    }
                }
            }
        }

        final Source source = new StreamSource(xslStream);
        final TransformerFactory transformerFactory = TransformerFactory.newInstance();

        ClassLoader customClassLoader = null;
        if (clazzProvided) {
            customClassLoader = ClassLoaderUtil.createClassLoader(installRoot, this.getClass()
                    .getClassLoader());
            Thread.currentThread().setContextClassLoader(customClassLoader);

        }

        ((XsltComponent) this.component).getTemplates().put(endpointName,
                transformerFactory.newTemplates(source));

        if (customClassLoader != null) {
            ((XsltComponent) this.component).getCustomizedClassLoader().put(endpointName,
                    customClassLoader);
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
            this.logger.finest("XSL stored with a customized classloader");
        }
    }

}
